#!/bin/sh

fmt()
{
    string="$1"
    direction="$2"
    fillchar="$3"
    size="$4"
    if [[ "${direction}" == "l" ]] || [[ "${direction}" == "L" ]] ; then
        while [[ ${#string} -lt ${size} ]] ; do
            string="${fillchar}${string}"
        done
        #string="${string: -${size}}"
    else
        while [[ ${#string} -lt ${size} ]] ; do
            string="${string}${fillchar}"
        done
        #string="${string:0:${size}}"
    fi
    echo "${string}"
}

log()
{
	
	echo $(fmt "$1" r '.' 70)$(fmt "$2" l '.' 10)
}

DDD_UDP_DIR=$(pwd)
DDD_UDP_SRC_DIR=$DDD_UDP_DIR/src
DDD_UDP_DRIVER_FILE=$DDD_UDP_DIR/build-$(uname -r)/nf_ddd_udp.ko
DDD_UDP_DRIVER_XZ_FILE=$DDD_UDP_DIR/build-$(uname -r)/nf_ddd_udp.ko.xz
DDD_UDP_MODULESD_FILE=$DDD_UDP_DIR/nf_ddd_udp.conf
DDD_UDP_SYSCTLD_FILE=$DDD_UDP_DIR/99-nf_ddd_udp.conf
NF_MODULES_PATH=/usr/lib/modules/$(uname -r)/kernel/net/ipv4/netfilter
SYSCTL_SERVICE=/usr/lib/systemd/system/systemd-sysctl.service

#
# check driver
#

if [ -e  $DDD_UDP_DRIVER_XZ_FILE ]; then

	log "$DDD_UDP_DRIVER_XZ_FILE" "ok"
else

	log "$DDD_UDP_DRIVER_XZ_FILE" "not exist"
    if [ -e  $DDD_UDP_DRIVER_FILE ]; then

	log "$DDD_UDP_DRIVER_FILE" "ok"
    else

	log "$DDD_UDP_DRIVER_FILE" "not exist"
	cd $DDD_UDP_SRC_DIR
	log "Compiling..."
	make
	RESULT_CODE=$?
	if [ $RESULT_CODE -ne 0 ]; then
	    log "compile failed"
	    exit 1
	else
	    log "compile ok"
	fi
	make clean
	cd $DDD_UDP_DIR
    fi
fi

#
# check moduled conf
#

if [ -e  $DDD_UDP_MODULESD_FILE ]; then

	log "$DDD_UDP_MODULESD_FILE" "ok"
else

	log "$DDD_UDP_MODULESD_FILE" "not exist"
	exit 0
fi

#
# check sysctl conf
#
if [ -e  $SYSCTL_SERVICE ]; then

	log "$SYSCTL_SERVICE" "ok"
else

	log "$SYSCTL_SERVICE" "not exist"
	exit 0
fi

#
# check sysctl service
#
if [ -e  $DDD_UDP_SYSCTLD_FILE ]; then

	log "$DDD_UDP_SYSCTLD_FILE" "ok"
else

	log "$DDD_UDP_SYSCTLD_FILE" "not exist"
	exit 0
fi

#
# cp module
#

if ls $NF_MODULES_PATH/*ko.xz 1> /dev/null 2>&1;
then
    /bin/cp -rf $DDD_UDP_DRIVER_XZ_FILE $NF_MODULES_PATH

    RESULT_CODE=$?
    if [ $RESULT_CODE -ne 0 ]; then

	log "$DDD_UDP_DRIVER_XZ_FILE" "cp failed"
	exit 0
    else
	log "$DDD_UDP_DRIVER_XZ_FILE" "cp ok"
    fi
fi
#
# For compatible with F20
#
if ls $NF_MODULES_PATH/*ko 1> /dev/null 2>&1;
then
    /bin/cp -rf $DDD_UDP_DRIVER_FILE $NF_MODULES_PATH

    RESULT_CODE=$?
    if [ $RESULT_CODE -ne 0 ]; then

	log "$DDD_UDP_DRIVER_FILE" "cp failed"
	exit 0
    else
	log "$DDD_UDP_DRIVER_FILE" "cp ok"
    fi
fi

#
# update dependence
#

depmod -a > /dev/null 2>&1

RESULT_CODE=$?
if [ $RESULT_CODE -ne 0 ]; then

	log "$DDD_UDP_DRIVER_FILE" "depmod failed"
	exit 0
else
	log "$DDD_UDP_DRIVER_FILE" "depmod ok"
fi

#
# cp module conf
#
/bin/cp -rf $DDD_UDP_MODULESD_FILE /etc/modules-load.d

RESULT_CODE=$?
if [ $RESULT_CODE -ne 0 ]; then

	log "$DDD_UDP_MODULESD_FILE" "cp failed"
	exit 0
else
	log "$DDD_UDP_MODULESD_FILE" "cp ok"
fi

#
# cp sysctl conf
#
/bin/cp -rf $DDD_UDP_SYSCTLD_FILE /etc/sysctl.d

RESULT_CODE=$?
if [ $RESULT_CODE -ne 0 ]; then

	log "$DDD_UDP_SYSCTLD_FILE" "cp failed"
	exit 0
else
	log "$DDD_UDP_SYSCTLD_FILE" "cp ok"
fi


#
# modify sysctl service
#

SYSCTL_SERVICE_AFTER=$(cat $SYSCTL_SERVICE | grep "After=")

if echo $SYSCTL_SERVICE_AFTER | grep -q systemd-modules-load.service
then

	log "systemd-sysctl.service load after systemd-modules-load.service" "ok"

else

	sed -i '/After/ s,$, systemd-modules-load.service,' $SYSCTL_SERVICE

	log "systemd-sysctl.service now load after systemd-modules-load.service" "ok"
fi
